--[[ Common ]]

--const
local MIN_VISIBLE_BUTTONS_SETTINGS_MODE = 1
FBC_BUTTON_PLACE_SIZE 	= 36
FBC_BUTTON_PLACE_OFFSET	= 4
FBC_FRAME_OFFSET 		= 5
FBC_ARROW_OFFSET 		= 4

--arrow button directions
FBC_DIR_UP		= 1
FBC_DIR_LEFT	= 2
FBC_DIR_DOWN	= 3
FBC_DIR_RIGHT	= 4

local SpellCacheToSlot = {}
local SpellCacheToId = {}

local function GetFullSpellName(name, rank)
	if not(rank) then
		rank = ""
	end
	if (name) then
		return name.."("..rank..")", name
	end
	return nil, nil
end

local function GetSpellNameBySlot(slot, book)
	--local item_type, id = GetSpellBookItemInfo(slot, BOOKTYPE_SPELL)
	--print("item_type "..tostring(item_type)..", id "..tostring(id))
	local name, rank = GetSpellBookItemName(slot, book)
	return GetFullSpellName(name, rank)
end

local function CacheSpellsByType(book, skillType, cache2slot, cache2id)
	local slot = 1
	while true do
		local item_type, id = GetSpellBookItemInfo(slot, book)
		if not(item_type) then
			break
		end
		if (item_type == skillType) then
			local nameRank, rank = GetSpellNameBySlot(slot, book)
			if not(nameRank) then
				break
			end
			cache2slot[nameRank] = {slot, book}
			cache2id[nameRank] = id
		end
		slot = slot + 1
	end
end

local SpellTypesToCache = {
	{
		["book"] = BOOKTYPE_SPELL,
		["skillType"] = "SPELL",
	}, -- [1]
	{
		["book"] = BOOKTYPE_PET,
		["skillType"] = "PETACTION",
	}, -- [2]
}

local CompanionTypes = {"MOUNT", "CRITTER"}

local function CacheCompanionSpells(compType, cache2slot, cache2id)
	local name, id
	for slot = 1, GetNumCompanions(compType) do
		--creatureID, creatureName, creatureSpellID, icon, issummoned
		_, name, id = GetCompanionInfo(compType, slot)
		local spellName = GetSpellInfo(id)
		--print("compType "..compType..", name "..name..", id "..id..", slot "..slot)
		cache2slot[spellName] = {slot, compType}
		cache2id[spellName] = id
	end
end

function FlyoutButton_CacheSpells()
	SpellCacheToSlot = {}
	SpellCacheToId = {}
	for k, v in ipairs(SpellTypesToCache) do
		CacheSpellsByType(v.book, v.skillType, SpellCacheToSlot, SpellCacheToId) 
	end
	for _, v in pairs(CompanionTypes) do
		CacheCompanionSpells(v, SpellCacheToSlot, SpellCacheToId) 
	end
end

function FlyoutButton_FindSpellSlot(spellName)
	local slot, slotType = nil, nil
	if SpellCacheToSlot[spellName] then
		slot, slotType = unpack(SpellCacheToSlot[spellName])
		--print("FlyoutButton_FindSpellSlot slot "..tostring(slot)..", slotType "..tostring(slotType))
	end
	return slot, slotType
end

function FlyoutButton_FindSpellId(spellName)
	--print("spellName "..tostring(spellName)..", book "..tostring(book))
	return SpellCacheToId[spellName]
end

function FlyoutButton_GetCompanionInfoByName(name, compType)
	local slot = unpack(SpellCacheToSlot[name])
	if slot then
		--creatureID, creatureName, creatureSpellID, icon, issummoned
		return GetCompanionInfo(compType, slot)
	else
		return nil
	end
end

function FlyoutButton_SetCursor(command, value, subValue, id)
	--print("SetCursor "..tostring(command)..", "..tostring(value)..", "..tostring(subValue))
	if not(command) then
		return
	end

	ClearCursor()
	if (command == "spell") then
		local id = FlyoutButton_FindSpellId(value, subValue)
		--print("FlyoutButton_SetCursor id "..tostring(id))
		if id then
			PickupSpell(id)
		else
			local slot = FlyoutButton_FindSpellSlot(value, subValue)
			--print("slot "..tostring(slot))
			if slot then
				PickupSpellBookItem(slot, subValue)
			end
		end
	elseif (command == "item") then
		PickupItem(value)	--itemID or "itemString" or "itemName" or "itemLink"
	elseif (command == "macro") then
		PickupMacro(value)
	elseif (command == "equipmentset") then
		PickupEquipmentSetByName(value)
	end
	FlyoutButton_GetCursorValues()
end

function FlyoutButton_GetCursorValues()
	local command, value, subValue, spellId = GetCursorInfo()
	--print("GetCursorValues "..tostring(command)..", "..tostring(value)..", "..tostring(subValue)..", "..tostring(spellId))
	if (command == "spell") then
		value = GetSpellNameBySlot(value, subValue)
		--print("value "..tostring(value))
	elseif (command == "item") or (command == "equipmentset") then
		--nothing to do
	elseif (command == "macro") then		
		subValue = value -- value is macro index	
		value = GetMacroInfo(value) --name, texture, body
	elseif (command == "companion") then
		local _, _, spellId = GetCompanionInfo(subValue, value)
		value = GetSpellInfo(spellId)
		command = "spell"
	end
	return command, value, subValue
end

function FlyoutButton_GetListButtonsCount(listTable)
	local count = 0
	if listTable then
		for i = 1, #listTable do
			if listTable[i].value then
				count = i
			end
		end
	end
	local settingsCount = count
	
	if FbcSettingsMode then
		if count < MIN_VISIBLE_BUTTONS_SETTINGS_MODE then
			settingsCount = MIN_VISIBLE_BUTTONS_SETTINGS_MODE
		else
			settingsCount = count + 1
		end
	end
	return count, settingsCount
end

